@@ -63,13 +63,14 @@ class Pulse(object):
     __slots__ = set(_readonly + _limits + ('rf', 'gr', 'dt', 'desc'))
 
     def __init__(
-            self,
-            rf: Optional[Tensor] = None, gr: Optional[Tensor] = None,
-            dt: Tensor = dt0,
-            gmax: Tensor = gmax0, smax: Tensor = smax0, rfmax: Tensor = rfmax0,
-            desc: str = "generic pulse",
-            device: torch.device = torch.device('cpu'),
-            dtype: torch.dtype = torch.float32):
+        self,
+        rf: Optional[Tensor] = None, gr: Optional[Tensor] = None,
+        dt: Tensor = dt0,
+        gmax: Tensor = gmax0, smax: Tensor = smax0, rfmax: Tensor = rfmax0,
+        desc: str = "generic pulse",
+        device: torch.device = torch.device('cpu'),
+        dtype: torch.dtype = torch.float32
+    ):
 
         assert(isinstance(device, torch.device) and
                isinstance(dtype, torch.dtype))
@@ -103,7 +104,7 @@ class Pulse(object):
     def __setattr__(self, k, v):
         if k in self._readonly:
             raise AttributeError(f"'Pulse' object attribute '{k}'"
-                                  " is read-only")
+                                 " is read-only")
 
         if k != 'desc':
             kw = {'device': self.device, 'dtype': self.dtype}
@@ -178,13 +179,15 @@ class Pulse(object):
         Usage:
             ``new_pulse = pulse.interpT(dt; kind)``
         Inputs:
-            - ``dt``: `(N,1)`, "Sec" simulation temporal step size, dwell time.
+            - ``dt``: `(1,)`, "Sec", new simulation dwell time.
             - ``kind``: str, passed to scipy.interpolate.interp1d.
         Outputs:
             - ``new_pulse``: mrphy.mobjs.Pulse object.
 
         .. note::
-            This method requires both `dt` and `self.dt` to be unique/global.
+            This method requires both `dt` and `self.dt` to be unique/global,
+            i.e., of shape ``(1,)``, which ensures pulse length to be the same
+            within a batch after interpolation.
         """
         assert(self.dt.numel() == dt.numel() == 1)
 
@@ -194,10 +197,11 @@ class Pulse(object):
 
         axis = 2  # Along temporal dimension
         dkw = {'device': self.device, 'dtype': self.dtype}
-        kw = {'axis':axis, 'kind':kind, 'copy':False, 'assume_sorted':True}
+        kw = {'axis': axis, 'kind': kind, 'copy': False, 'assume_sorted': True}
 
         f_np = lambda x: x.detach().cpu().numpy()  # noqa: E731
-        f_0 = lambda x: np.dstack((np.zeros_like(x[:,:,[0]]), x))  # noqa: E731
+        f_0 = lambda x: np.dstack((np.zeros_like(x[:, :, [0]]),  # noqa: E731
+                                   x))
 
         # convert to np array, then prepend 0's.
         rf_np, gr_np = f_0(f_np(self.rf)), f_0(f_np(self.gr))
@@ -227,12 +231,10 @@ class Pulse(object):
         Outputs:
             - ``new_pulse``: mrphy.mobjs.Pulse object.
         """
-        if (self.device != device) or (self.dtype != dtype):
-            return Pulse(self.rf, self.gr, dt=self.dt, desc=self.desc,
-                         device=device, dtype=dtype)
-        else:
+        if self.device == device and self.dtype == dtype:
             return self
-        return
+        return Pulse(self.rf, self.gr, dt=self.dt, desc=self.desc,
+                     device=device, dtype=dtype)
 
 
 class SpinArray(object):
@@ -296,13 +298,14 @@ class SpinArray(object):
     __slots__ = set(_readonly + _compact)
 
     def __init__(
-            self, shape: tuple, mask: Optional[Tensor] = None,
-            T1: Optional[Tensor] = None, T1_: Optional[Tensor] = None,
-            T2: Optional[Tensor] = None, T2_: Optional[Tensor] = None,
-            γ: Optional[Tensor] = None,  γ_: Optional[Tensor] = None,
-            M: Optional[Tensor] = None,  M_: Optional[Tensor] = None,
-            device: torch.device = torch.device('cpu'),
-            dtype: torch.dtype = torch.float32):
+        self, shape: tuple, mask: Optional[Tensor] = None,
+        T1: Optional[Tensor] = None, T1_: Optional[Tensor] = None,
+        T2: Optional[Tensor] = None, T2_: Optional[Tensor] = None,
+        γ: Optional[Tensor] = None,  γ_: Optional[Tensor] = None,
+        M: Optional[Tensor] = None,  M_: Optional[Tensor] = None,
+        device: torch.device = torch.device('cpu'),
+        dtype: torch.dtype = torch.float32
+    ):
 
         mask = (torch.ones((1,)+shape[1:], dtype=torch.bool, device=device)
                 if mask is None else mask.to(device=device))
@@ -357,7 +360,7 @@ class SpinArray(object):
     def __setattr__(self, k_, v_):
         if (k_ in self._readonly) or (k_+'_' in self._readonly):
             raise AttributeError(f"'SpinArray' object attribute '{k_}'"
-                                  " is read-only")
+                                 " is read-only")
 
         # Transfer ``v_`` to ``kw`` before ``extract`
         kw = {'device': self.device, 'dtype': self.dtype}
@@ -379,29 +382,36 @@ class SpinArray(object):
         return
 
     def applypulse(
-            self, pulse: Pulse, doEmbed: bool = False, doRelax: bool = True,
-            loc: Optional[Tensor] = None, loc_: Optional[Tensor] = None,
-            Δf: Optional[Tensor] = None, Δf_: Optional[Tensor] = None,
-            b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
-            ) -> Tensor:
+        self, pulse: Pulse,
+        doEmbed: bool = False, doRelax: bool = True, doUpdate: bool = False,
+        loc: Optional[Tensor] = None, loc_: Optional[Tensor] = None,
+        Δf: Optional[Tensor] = None, Δf_: Optional[Tensor] = None,
+        b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
+    ) -> Tensor:
         r"""Apply a pulse to the spinarray object
 
         Typical usage:
             ``M = spinarray.applypulse(pulse; loc, doEmbed=True, doRelax, ``\
-            ``Δf, b1Map)``
+            ``doUpdate, Δf, b1Map)``
             ``M_ = spinarray.applypulse(pulse; loc_, doEmbed=False, `` \
-            ``doRelax, Δf_, b1Map_)``
+            ``doRelax, doUpdate, Δf_, b1Map_)``
         Inputs:
             - ``pulse``: mrphy.mobjs.Pulse.
             - ``loc`` ⊻ ``loc_``: `(N,*Nd ⊻ nM,xyz)`, "cm", locations.
         Optionals:
             - ``doEmbed``: [t/F], return ``M`` or ``M_``
             - ``doRelax``: [T/f], do relaxation during Bloch simulation.
+            - ``doUpdate``: [t/F], update ``self.M_``
             - ``Δf``⊻ ``Δf_``: `(N,*Nd ⊻ nM)`, "Hz", off-resonance.
             - ``b1Map`` ⊻ ``b1Map_``: `(N,*Nd ⊻ nM,xy,(nCoils))`, transmit \
               sensitivity.
         Outputs:
             - ``M`` ⊻ ``M_``: `(N,*Nd ⊻ nM,xyz)`
+
+        .. note::
+            When ``doUpdate == True and doEmbed == False``, the output compact
+            magnetization Tensor is a reference to ``self.M_``, and needs
+            caution when being accessed.
         """
         assert ((loc_ is None) != (loc is None))  # XOR
         loc_ = (loc_ if loc is None else self.extract(loc))
@@ -424,6 +434,8 @@ class SpinArray(object):
         kw_bsim['dt'] = pulse.dt
 
         M_ = sims.blochsim(self.M_, beff_, **kw_bsim)
+        if doUpdate:
+            self.M_ = M_
         M_ = (self.embed(M_) if doEmbed else M_)
         return M_
 
@@ -556,11 +568,11 @@ class SpinArray(object):
         return self.mask.numel()
 
     def pulse2beff(
-            self, pulse: Pulse, doEmbed: bool = False,
-            loc: Optional[Tensor] = None, loc_: Optional[Tensor] = None,
-            Δf: Optional[Tensor] = None, Δf_: Optional[Tensor] = None,
-            b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
-            ) -> Tensor:
+        self, pulse: Pulse, doEmbed: bool = False,
+        loc: Optional[Tensor] = None, loc_: Optional[Tensor] = None,
+        Δf: Optional[Tensor] = None, Δf_: Optional[Tensor] = None,
+        b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
+    ) -> Tensor:
         r"""Compute B-effective of ``pulse`` with the spinarray's parameters
 
         Typical usage:
@@ -603,8 +615,11 @@ class SpinArray(object):
         """
         return self.shape
 
-    def to(self, device: torch.device = torch.device('cpu'),
-           dtype: torch.dtype = torch.float32) -> SpinArray:
+    def to(
+        self,
+        device: torch.device = torch.device('cpu'),
+        dtype: torch.dtype = torch.float32
+    ) -> SpinArray:
         r"""Duplicate the object to the prescribed device with dtype
 
         Usage:
@@ -655,15 +670,16 @@ class SpinCube(object):
     __slots__ = set(_readonly+_compact+('fov', 'ofst'))
 
     def __init__(
-            self, shape: tuple, fov: Tensor, mask: Optional[Tensor] = None,
-            ofst: Tensor = tensor([[0., 0., 0.]]),
-            Δf: Optional[Tensor] = None, Δf_: Optional[Tensor] = None,
-            T1: Optional[Tensor] = None, T1_: Optional[Tensor] = None,
-            T2: Optional[Tensor] = None, T2_: Optional[Tensor] = None,
-            γ: Optional[Tensor] = None,  γ_: Optional[Tensor] = None,
-            M: Optional[Tensor] = None,  M_: Optional[Tensor] = None,
-            device: torch.device = torch.device('cpu'),
-            dtype: torch.dtype = torch.float32):
+        self, shape: tuple, fov: Tensor, mask: Optional[Tensor] = None,
+        ofst: Tensor = tensor([[0., 0., 0.]]),
+        Δf: Optional[Tensor] = None, Δf_: Optional[Tensor] = None,
+        T1: Optional[Tensor] = None, T1_: Optional[Tensor] = None,
+        T2: Optional[Tensor] = None, T2_: Optional[Tensor] = None,
+        γ: Optional[Tensor] = None,  γ_: Optional[Tensor] = None,
+        M: Optional[Tensor] = None,  M_: Optional[Tensor] = None,
+        device: torch.device = torch.device('cpu'),
+        dtype: torch.dtype = torch.float32
+    ):
         sp = SpinArray(shape, mask, T1=T1, T1_=T1_, T2=T2, T2_=T2_, γ=γ, γ_=γ_,
                        M=M, M_=M_, device=device, dtype=dtype)
         super().__setattr__('spinarray', sp)
@@ -698,7 +714,7 @@ class SpinCube(object):
     def __setattr__(self, k_, v_):
         if (k_ in self._readonly) or (k_+'_' in self._readonly):
             raise AttributeError(f"'SpinCube' object attribute '{k_}'"
-                                  " is read-only")
+                                 " is read-only")
 
         sp = self.spinarray
         if k_ in SpinArray.__slots__ or k_+'_' in SpinArray.__slots__:
@@ -752,9 +768,10 @@ class SpinCube(object):
         return
 
     def applypulse(
-            self, pulse: Pulse, doEmbed: bool = False, doRelax: bool = True,
-            b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
-            ) -> Tensor:
+        self, pulse: Pulse,
+        doEmbed: bool = False, doRelax: bool = True, doUpdate: bool = False,
+        b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
+    ) -> Tensor:
         r"""Apply a pulse to the spincube object
 
         Usage:
@@ -775,8 +792,9 @@ class SpinCube(object):
         b1Map_ = (b1Map_ if b1Map is None else self.extract(b1Map))
 
         return self.spinarray.applypulse(pulse, doEmbed=doEmbed,
-                                         doRelax=doRelax, Δf_=self.Δf_,
-                                         loc_=self.loc_, b1Map_=b1Map_)
+                                         doRelax=doRelax, doUpdate=doUpdate,
+                                         Δf_=self.Δf_, loc_=self.loc_,
+                                         b1Map_=b1Map_)
 
     def asdict(self, toNumpy: bool = True, doEmbed: bool = True) -> dict:
         r"""Convert mrphy.mobjs.SpinCube object to dict
@@ -882,9 +900,10 @@ class SpinCube(object):
         return self.spinarray.numel()
 
     def pulse2beff(
-            self, pulse: Pulse, doEmbed: bool = False,
-            b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
-            ) -> Tensor:
+        self, pulse: Pulse,
+        doEmbed: bool = False,
+        b1Map: Optional[Tensor] = None, b1Map_: Optional[Tensor] = None
+    ) -> Tensor:
         r"""Compute B-effective of ``pulse`` with the spincube's parameters
 
         Typical usage:
@@ -913,8 +932,11 @@ class SpinCube(object):
         """
         return self.spinarray.size()
 
-    def to(self, device: torch.device = torch.device('cpu'),
-           dtype: torch.dtype = torch.float32) -> SpinCube:
+    def to(
+        self,
+        device: torch.device = torch.device('cpu'),
+        dtype: torch.dtype = torch.float32
+    ) -> SpinCube:
         r"""Duplicate the object to the prescribed device with dtype
 
         Usage:
@@ -925,13 +947,11 @@ class SpinCube(object):
         Outputs:
             - ``new_spincube``: mrphy.mobjs.SpinCube object.
         """
-        if (self.device != device) or (self.dtype != dtype):
-            return SpinCube(self.shape, self.fov, ofst=self.ofst, Δf_=self.Δf_,
-                            T1_=self.T1_, T2_=self.T2_, γ_=self.γ_, M_=self.M_,
-                            device=device, dtype=dtype)
-        else:
+        if self.device == device and self.dtype == dtype:
             return self
-        return
+        return SpinCube(self.shape, self.fov, ofst=self.ofst, Δf_=self.Δf_,
+                        T1_=self.T1_, T2_=self.T2_, γ_=self.γ_, M_=self.M_,
+                        device=device, dtype=dtype)
 
 
 class SpinBolus(SpinArray):
